% FUNCTION [NEW_S, GAP_CONTENTS] = EXCISE_BREATHS_AND_PAUSES(S,SR,WIN_MS,AMP_THRESH,GAP_LEN_THRESH,FNAME,OUTPUT_DIR, DISPLAY_FIGURES)
%
% Excises low amplitude sections of a waveform S, typically breaths and
% pauses in speech. Returns a waveform NEW_S with the low amplitude
% sections removed (with fades applied to the remaining segments), as well
% as a waveform GAP_CONTENTS that has the excised segments.
%
% WIN_MS is the duration in ms of the segments over which the amplitude is
% computed.
%
% Sections are removed if they consist of at least GAP_LEN_THRESH segments
% each of whose rms level is below that of the maximum segment level by
% AMP_THRESH dB.
%
% Setting AMP_THRESH and GAP_LEN_THRESH at -35 and 8 seems to be about
% right for most speech recordings.
%
% If FNAME is nonempty, NEW_S and GAP_CONTENTS as well as S are written to
% WAV files.
%
% If DISPLAY_FIGURES is set to 1, figures are generated and saved that show
% the amplitude and gap statistics for S.
%

%
% March 4 2013 - Josh McDermott (jhm@mit.edu)

function [new_s, gap_contents] = excise_breaths_and_pauses(s,sr,win_ms,amp_thresh,gap_len_thresh,fname,output_dir, display_figures)

if nargin<3
    win_ms = 30;
end
if nargin<4
    amp_thresh = -35;
end
if nargin<5
    gap_len_thresh = 8;
end
if nargin<6
    fname = [];
end
if nargin<7
    output_dir = '.';
end
if nargin<8
    display_figures = 0;
end

ramp_length_ms = 10;

win_length_smp = round(win_ms/1000*sr);
n_seg = floor(length(s)/win_length_smp);
s = s(1:n_seg*win_length_smp);
if size(s,2)==1 %make row vector
    s = s';
end

index=0;
hann_win = hann(ones(1,win_length_smp),ramp_length_ms,sr);
for k=1:n_seg
    %if rem(k,10000)==0
    %    fprintf('starting segment %d of %d...\n',k,n_seg);
    %end
    segment_power(k) = rms(s(index+1:index+win_length_smp));
    index = index + win_length_smp;
end

seg_power_dB = 20*log10(segment_power/max(segment_power));
below_thresh = -1;
gap_num=0;
for k=1:n_seg
    if seg_power_dB(k) < amp_thresh
        if below_thresh~=1
            gap_num = gap_num+1;
            gap_start(gap_num) = k;
            below_thresh=1;
            gap_len(gap_num)=0;
        end
        gap_len(gap_num) = gap_len(gap_num)+1;
    else
        if below_thresh~=0
            below_thresh=0;
        end
    end
end

[n,x]=hist(gap_len,[1:150]);
if display_figures
    figure;
    hold on
    plot(x,n,'r*');
    plot(x,fliplr(cumsum(fliplr(n))),'o');
    xlabel('Gap Length (30 ms windows)');
    temp = get(gca,'YLim');
    plot(gap_len_thresh*[1 1],temp,'k');
    legend('Counts','Cumulative','Gap Threshold');
    saveas(gcf, [output_dir '/' fname '_gap_length_hist.pdf']);
    close
end

gap_start_smp = win_length_smp*(gap_start(find(gap_len>=gap_len_thresh))-1)+1;
gap_end_smp = win_length_smp*(gap_start(find(gap_len>=gap_len_thresh))+gap_len(find(gap_len>=gap_len_thresh))-1);

hann_win_left = [hann_win(1:win_length_smp/2) ones(1,win_length_smp/2)];
hann_win_right = [ones(1,win_length_smp/2) hann_win(win_length_smp/2+1:win_length_smp)];
indices2=ones(size(s));
new_s = s;
for k=1:length(gap_start_smp)
    indices2(gap_start_smp(k)+win_length_smp:gap_end_smp(k)-win_length_smp)=0;
    new_s(gap_start_smp(k):gap_start_smp(k)+win_length_smp-1) = s(gap_start_smp(k):gap_start_smp(k)+win_length_smp-1).*hann_win_right;
    new_s(gap_end_smp(k)-win_length_smp+1:gap_end_smp(k)) = s(gap_end_smp(k)-win_length_smp+1:gap_end_smp(k)).*hann_win_left;
end

new_s=new_s(find(indices2));
gap_contents = s(find(indices2==0));

if display_figures
    figure;
    plot([1:length(seg_power_dB)]*(win_ms/1000),seg_power_dB,'b');
    hold on
    plot([1 length(seg_power_dB)]*(win_ms/1000), [amp_thresh amp_thresh],'r');
    temp2 = gap_start(find(gap_len>=gap_len_thresh));
    temp3 = gap_start(find(gap_len>=gap_len_thresh))+gap_len(find(gap_len>=gap_len_thresh))-1;
    for k=1:length(temp2)
        plot([temp2(k):temp3(k)]*(win_ms/1000),seg_power_dB(temp2(k):temp3(k)),'r');
    end
    xlabel('Seconds');
    ylabel('Power (dB)');
    saveas(gcf, [output_dir '/' fname '_seg_power.pdf']);
    close
end

if ~isempty(fname)
    wavwrite(s,sr,[output_dir '/' fname '_original.wav']);
    wavwrite(new_s,sr,[output_dir '/' fname '_nogaps.wav']);
    wavwrite(gap_contents,sr,[output_dir '/' fname '_inthegaps.wav']);
    save([output_dir '/' fname '_gap_stats'], 'seg_power_dB', 'gap_len', 'gap_start_smp', 'gap_end_smp');
end