% FUNCTION [QUILTED_S, FINAL_SEG_ORDER, SOURCE_SEG_CHANGES, QUILT_SEG_CHANGES, KL] = ...
%    GENERATE_QUILT(SOURCE_S, WIN_MS, SEGMENT_SIZE_IN_WIN, STIM_LENGTH_IN_SEC, P)
%
% -divides signal SOURCE_S into segments of length SEGMENT_SIZE_IN_WIN*WIN_MS
% -returns QUILTED_S, a "quilt" generated by concatenating segments of SOURCE_S in
% the order specified in FINAL_SEG_ORDER. Concatenation is done via
% pitch-synchronous overlap-add, appending the segments so as to maximize
% the cross-correlation between segment waveforms.
%
% -segment order is generated by starting with a randomly selected segment
% and then choosing successive segments to approximate the change in the
% cochleogram that occurs from segment to segment in the original signal
% SOURCE_S (whilst avoiding the segment-to-segment pairings present in the
% source signal).
%
% -in this version, the segment change between segment j and the next segment
% is chosen to be as close as possible to the segment change for segment j
% in the original signal (so as to match the segment-to-segment statistics)
%
% SOURCE_SEG_CHANGES contains the magnitude of the cochleogram change
% between each successive pair of segments in the original signal;
% QUILT_SEG_CHANGES contains the change between successive segments in
% QUILTED_S. KL gives the Kullback-Leibler divergence between the
% histograms of SOURCE_SEG_CHANGES and QUILT_SEG_CHANGES.
%
% The structure P contains the parameters of the auditory filter bank used
% to compute the cochleogram from which segment-to-segment changes are
% measured:
%
% P.filt_density should be 1 for regular filterbank, 2 for a filterbank 4x overcomplete
% P.N_audio_channel - the number of frequency channels, e.g. 30
% P.audio_low_lim_Hz - the lower cutoff of the lowest filter
% P.audio_high_lim_Hz - the upper cutoff of the highest filter
% P.audio_sr - the sampling rate
%
% March 1 2013 - Josh McDermott (jhm@mit.edu)

function [quilted_s, final_seg_order, source_seg_changes, quilt_seg_changes, kl] = ...
    generate_quilt(source_s, win_ms, segment_size_in_win, stim_length_in_sec, P)


win_length_smp = round(win_ms/1000*P.audio_sr);
concatenation_method = 3;

if size(source_s,1)>2 %make s a row vector
    source_s=source_s';
end
if size(source_s,1)==2 %stereo
    source_s = source_s(:,1); %take right channel
end
source_s=source_s/rms(source_s)*.03;

[final_seg_order, source_seg_changes, quilt_seg_changes] = ...
    make_seg_order(source_s, win_ms, segment_size_in_win, stim_length_in_sec, P);

[quilted_s,shifts] = reorder_segs(source_s, win_length_smp*segment_size_in_win, ...
    final_seg_order, concatenation_method, win_length_smp, win_length_smp/2);

[n1,bins] = hist(source_seg_changes, 40);
[n2,bins] = hist(quilt_seg_changes, bins);
n1 = n1 + .001; n2 = n2 + .001;
p1 = n1/sum(n1); p2 = n2/sum(n2);

kl = sum(p1.*log(p1./p2));

