% FUNCTION [FINAL_SEG_ORDER, SOURCE_SEG_CHANGES, QUILT_SEG_CHANGES] = MAKE_SEG_ORDER(SOURCE_S, WIN_MS, SEGMENT_SIZE_IN_WIN, STIM_LENGTH_IN_SEC, P)
%
% -takes signal SOURCE_S as input
% -divides into segments of length SEGMENT_SIZE_IN_WIN*WIN_MS
% -returns a segment ordering FINAL_SEG_ORDER from which a sound quilt can
% be generated using the function REORDER_SEGS
%
% -segment order is generated by starting with a randomly selected segment
% and then choosing successive segments to approximate the change in the
% cochleogram that occurs from segment to segment in the original signal
% SOURCE_S (whilst avoiding the segment-to-segment pairings present in the
% source signal).
% -in this version, the segment change between segment j and the next segment
% is chosen to be as close as possible to the segment change for segment j
% in the original signal (so as to match the segment-to-segment statistics)
%
% SOURCE_SEG_CHANGES contains the magnitude of the cochleogram change
% between each successive pair of segments in the original signal;
% QUILT_SEG_CHANGES contains the change between successive segments given
% the order specified in FINAL_SEG_ORDER.
%
% The structure P contains the parameters of the auditory filter bank used
% to compute the cochleogram from which segment-to-segment changes are
% measured.
%
% originally written 7/7/2010 by Josh McDermott
% comments added 3/1/2013 by Josh McDermott (jhm@mit.edu)
%
% updated 7/16/2016 to fix two bugs: Previously source_seg_changes computed
% only n_seg_total-2 boundaries between segments instead of n_seg_total-1
% boundaries, and previously it was possible to sample the last segment
% which then threw an error when the algorithm attempted to find the
% boundary for this segment. Thanks to Sam Norman-Haignere for catching
% these.

function [final_seg_order, source_seg_changes, quilt_seg_changes] = make_seg_order(source_s, win_ms, segment_size_in_win, stim_length_in_sec, P)

win_length_smp = round(win_ms/1000*P.audio_sr);
n_win_total = floor(length(source_s)/win_length_smp);
n_win_total = n_win_total - rem(n_win_total,segment_size_in_win); %total number of sub-segments in source signal
n_seg_total = n_win_total/segment_size_in_win;
desired_length_in_seg = ceil(stim_length_in_sec / (win_ms/1000*segment_size_in_win));

if P.filt_density==1
    [filts,Hz_cutoffs,freqs]=make_erb_cos_filters(length(source_s)*2,P.audio_sr,P.N_audio_channel,P.audio_low_lim_Hz,P.audio_high_lim_Hz);
elseif P.filt_density==2
    [filts,Hz_cutoffs,freqs]=make_erb_cos_filts_quadruple2(length(s)*2,P.audio_sr,P.N_audio_channel,P.audio_low_lim_Hz,P.audio_high_lim_Hz);
end
subbands = generate_subbands_zp(source_s', filts); %needs a column vector
subband_envs = abs(hilbert(subbands)).^.3; %apply compression

s_segments=[];
index=0;
segment_window=ones(1,win_length_smp);
%divide source into segments
for k=1:n_seg_total
    if rem(k,100)==0
        %fprintf('starting segment %d of %d...\n',k,n_seg);
    end
    temp=[];
    for w = 1:segment_size_in_win
        temp = [temp source_s(index+1:index+win_length_smp)];
        index=index+win_length_smp;
    end
    s_segments(k,:) = temp;
    %compute segment-to-segment change in source signal - between borders
    %of adjacent segments (borders are the entire segment if
    %segment_size_in_win is 1)
    %Note that this measure of change approximates the change in the
    %cochlear excitation pattern because the window size typically used is
    %short (30 ms). It would not make sense if the window were much longer
    %than this. This will probably be updated in future versions.
    if k<=n_seg_total-1
        source_seg_changes(k) = mean(mean((subband_envs(index-win_length_smp+1:index,:)-...
            subband_envs(index+1:index+win_length_smp,:)).^2,2).*segment_window');
    end
end

%select first quilt patch at random
current_seg_index = ceil(rand*(n_seg_total-1)); %don't want to select last segment because it has no entry in source_seg_changes
final_seg_order=[current_seg_index];
for k = 2:desired_length_in_seg    
    %compute difference between borders of current segment and all others
    segs_to_avoid = [final_seg_order current_seg_index-1:current_seg_index+1 1 2]; %sample without replacement, avoid neighbors in source signal
    for j=1:n_seg_total-2
        if ~ismember(j,segs_to_avoid)
            current_seg_start_smp = (current_seg_index-1)*win_length_smp*segment_size_in_win + 1;
            current_seg_border_start_smp = current_seg_start_smp+win_length_smp*(segment_size_in_win-1);
            current_seg_border_end_smp = current_seg_start_smp+win_length_smp*segment_size_in_win -1;
            candidate_seg_start_smp = (j-1)*win_length_smp*segment_size_in_win + 1;
            temp = mean(mean((subband_envs(current_seg_border_start_smp:current_seg_border_end_smp,:)-...
                subband_envs(candidate_seg_start_smp:candidate_seg_start_smp+win_length_smp-1,:)).^2,2).*segment_window');
            dist(j) = temp;
        else
            dist(j) = 100;
            j=j+1;
        end
    end
    %find segment in set that that produces change closest to size of that
    %in source signal
    temp = abs(dist-source_seg_changes(current_seg_index));
    seg_to_use = find(temp==min(temp));
    current_seg_index = seg_to_use;
    quilt_seg_changes(k-1)=dist(seg_to_use);
    final_seg_order(k) = seg_to_use;
end



