function [y,shifts] = reorder_segs(d, hop_pts, order, method, olap_pts, max_shift)
% [y,shifts] = reorder_segs(d, hop_pts, order, method, olap_pts, max_shift)
%   Take a waveform <d> reorder it in blocks <hop_pts> apart, in
%   the order specified in <order> (indexing from 1).  Implements 3
%   methods:
%   <method = 1>: simply move blocks of hop_pts around
%   <method = 2>: use blocks of <hop_pts+olap_pts> with
%        raised-cosine cross-fades in the <olap_pts> of overlap between
%        between adjacent blocks
%   <method = 3>: overlap-add like method 2, but additionally shift
%        the point from which each new window is taken by up to 
%        +/- <max_shift> points to maximize the cross-correlation
%        within the overlap region (and hence the continuity of the
%        synthesis).
%
% 2011-02-02 Dan Ellis dpwe@ee.columbia.edu
%
% 2016-07-14 -- modified to ensure max_shift is an integer (thanks to
% Sam Norman-Haignere for catching this) -- Josh McDermott

if nargin < 4; method = 1; end
if nargin < 5; olap_pts = hop_pts; end
if nargin < 6; max_shift = hop_pts/2; end
  
% d must be a column (for flipud below)
if size(d,1) == 1
  d = d';
end

max_shift = floor(max_shift);

shifts = zeros(1,length(order));

if method == 1
  % Rectangular windows - existing approach
  ld = length(d);

  nwins = ceil(ld/hop_pts);

  segs = reshape([d;zeros(nwins*hop_pts - ld,1)],hop_pts,nwins);

  y = segs(:,order);
  y = y(:);

else
  
  % common setup for 2 and 3
  
  % overlap extend to left and to right
  half_olap = floor(olap_pts/2);
  half_olap_2 = olap_pts-half_olap;

  % pre-initialize output, including overlap beyond end
  y = zeros(hop_pts*length(order)+half_olap_2,1);
  
  % actual window is the non-overlapping part, plus half an overlap
  % at each end
  win_pts = hop_pts + half_olap + half_olap_2;
  
  %  win = hanning(win_pts);
  win = [.5*(1-cos(pi*[0:olap_pts-1]'/olap_pts));ones(hop_pts-olap_pts,1); ...
         .5*(cos(pi*[0:olap_pts-1]'/olap_pts)+1)];
  %plot(win) 
  % pre-initialize output, including overlap beyond end
  y = zeros(hop_pts*length(order)+half_olap_2,1);
  
  % now, with or without phase fine-tuning
  if method == 2
    % No attempt to align, but include overlap ramps
    Yw = diag(win)*frame([zeros(half_olap,1);d;zeros(half_olap_2,1)], ...
                         win_pts, hop_pts);
    Yw = Yw(:,order);
    y = ola(Yw,hop_pts);
    % remove padding
    y = y((half_olap + 1):(end-half_olap_2));
  
  elseif method == 3
    % ramping, plus cross-correlation optimization of alignment

    % We construct the output frame by frame
    % We choose the precise alignment of the next frame by
    % cross-correlating in the overlap region
  
    % Seed with the first window
    % starting point of frame requested by <order>
    dbase = (order(1)-1)*hop_pts;
    threeqwinix = 1:(hop_pts+half_olap_2);
    y(threeqwinix) = d(dbase + threeqwinix) .* win(half_olap+threeqwinix);
  
    % indices of overlap region, relative to first "real" point
    olap_ix = (-half_olap+1):half_olap_2;
    % indices of full window, relative to first "real" point
    win_ix  = (-half_olap+1):(hop_pts+half_olap_2);

    for f = 2:length(order)
      % starting point of frame requested by <order>
      dbase = (order(f)-1)*hop_pts;
      % starting point of where it's going to go (ignoring overlap)
      ybase = (f-1)*hop_pts;
      olap_region = y(ybase+olap_ix);
      % apply second window (that will be applied to final overlap choice)
      olap_region = olap_region.*win(1:length(olap_region));
      % cross-correlate
      % what is the block of candidate overlap points from original signal?
      cand_block = d(dbase+[(-half_olap-max_shift+1):(half_olap_2+max_shift)]);
      xc = conv(cand_block, flipud(olap_region))/ ...
           sqrt(sum(olap_region.^2));  % normalize in case we want to plot
%
%      % choose best point, express relative to ideal point
%      shift = find(xc==max(xc))-(length(olap_region)-1)-max_shift -1;
%
      % Instead, first trim off the ends...
      xc = xc(length(olap_region)+[1:(1+2*max_shift)]);
      % ... then find shift in that part only
      shift = find(xc == max(xc))-max_shift -1;
%
      % overlap-add at that point
      op_range = ybase + win_ix;
      overlap_part = win .* d(dbase + shift+win_ix);
%      % DEBUG
%      if f>0
%        % check: do the parts we're about to overlap really align in phase?
%        plot(win_ix, y(op_range), ...
%             win_ix, overlap_part);
%        title(['Frame ',num2str(f)]);
%        pause
%      end
      y(op_range) = y(op_range) + overlap_part;
      shifts(f) = shift;
    end
  
    % trim fade out from the end
    y = y(1:end-half_olap_2);
  end % method 2/3
  
end % method 1/not 1
